@@ -1,9 +1,10 @@
 #include "header.h"
 
-void smithfull3d(double *data, double *distVec, int *nSite,
-		 int *nObs, double *locs, double *scales, double *shapes,
-		 double *cov11, double *cov12, double *cov13, double *cov22,
-		 double *cov23, double *cov33, int *fitmarge, double *dns){
+void smithfull3d(double *data, double *distVec, int *nSite, int *nObs,
+		 int *weighted, double *weights, double *locs, double *scales,
+		 double *shapes, double *cov11, double *cov12, double *cov13,
+		 double *cov22, double *cov23, double *cov33, int *fitmarge,
+		 double *dns){
   //This is the Smith model - 3d case. It's a wrapper to several
   //sub-functions. It's named xxxfull as it either assume that the
   //margins are unit Frechet, or the GEV parameters are estimated at
@@ -42,21 +43,29 @@ void smithfull3d(double *data, double *distVec, int *nSite,
     if (*dns != 0.0)
       return;
     
-    *dns = lpliksmith(frech, mahalDist, jac, *nObs, *nSite);
+    if (*weighted)
+      *dns = wlpliksmith(frech, mahalDist, jac, *nObs, *nSite, weights);
+
+    else
+      *dns = lpliksmith(frech, mahalDist, jac, *nObs, *nSite);
   }
   
   else {
     for (i=0;i<(*nSite * *nObs);i++)
       jac[i] = 0.0;    
 
-    *dns = lpliksmith(data, mahalDist, jac, *nObs, *nSite);
+    if (*weighted)
+      *dns = wlpliksmith(data, mahalDist, jac, *nObs, *nSite, weights);
+
+    else
+      *dns = lpliksmith(data, mahalDist, jac, *nObs, *nSite);
   }
 
   return;
 }
 
-void smithdsgnmat3d(double *data, double *distVec, int *nSite, int *nObs, 
-		    double *locdsgnmat, double *locpenmat, int *nloccoeff,
+void smithdsgnmat3d(double *data, double *distVec, int *nSite, int *nObs, int *weighted,
+		    double *weights, double *locdsgnmat, double *locpenmat, int *nloccoeff,
 		    int *npparloc, double *locpenalty, double *scaledsgnmat,
 		    double *scalepenmat, int *nscalecoeff, int *npparscale,
 		    double *scalepenalty, double *shapedsgnmat, double *shapepenmat,
@@ -100,7 +109,11 @@ void smithdsgnmat3d(double *data, double *distVec, int *nSite, int *nObs,
   if (*dns != 0.0)
     return;
 
-  *dns = lpliksmith(frech, mahalDist, jac, *nObs, *nSite);
+  if (*weighted)
+    *dns = wlpliksmith(frech, mahalDist, jac, *nObs, *nSite, weights);
+
+  else
+    *dns = lpliksmith(frech, mahalDist, jac, *nObs, *nSite);
     
   //Stage 5: Removing the penalizing terms (if any)
   // 1- For the location parameter
